# -*- coding: utf-8 -*-
import base64
import os
from flask import Flask as _Flask, request, json, jsonify
from flask.json import JSONEncoder as _JSONEncoder
from datetime import date
from bson import ObjectId
from werkzeug.exceptions import InternalServerError, HTTPException


class JSONEncoder(_JSONEncoder):
    def default(self, o):
        if hasattr(o, 'keys') and hasattr(o, '__getitem__'):
            return dict(o)
        if isinstance(o, date):
            return o.strftime('%Y-%m-%d')
        if isinstance(o, ObjectId):
            return str(o)
        raise InternalServerError("对象序列发时发生不可预期的错误")


class Flask(_Flask):
    """
    修复中文序列化时的乱码
    """
    json_encoder = JSONEncoder


def get_token():
    headers = request.headers
    jwt = headers['AUTHORIZATION'].split(' ')[1]
    data = base64.urlsafe_b64decode(jwt.split(".")[1] + '=' * (4 - len(jwt.split(".")[1]) % 4))
    return json.loads(data)


def result(result=None, success=True, message=u'成功'):
    """
    返回JSON化的结果集
    :param result:
    :param success:
    :param message:
    :return:
    """
    return jsonify({'success': success,
                    'message': message,
                    'result': result if result is not None else {}})


def fail(message=u'失败'):
    """
    返回错误的JSON化结果集
    :param message:
    :return:
    """
    return result(message=message, success=False)


def create_flask(name, default_blueprint=None):
    app = Flask(name, instance_relative_config=True)
    app.config.from_object('config.' + os.environ['FLASK_ENV'])
    app.config.from_pyfile('config.py')
    if default_blueprint is not None:
        app.register_blueprint(default_blueprint, url_prefix='/')
    app.register_error_handler(flask_error_handler)
    return app


def flask_error_handler(e):
    return jsonify({
        'code': e.code if isinstance(e, HTTPException) else 500,
        'msg': e.description
    })



# coding: utf-8
import click
import os
from shutil import copytree, copy2, copystat


@click.command()
def execute():
    dst = os.getcwd()
    src = os.path.join(os.path.dirname(__file__), "templates")
    names = os.listdir(src)

    for name in names:
        srcname = os.path.join(src, name)
        dstname = os.path.join(dst, name)


        if os.path.isdir(srcname):
            copytree(srcname, dstname)
        else:
            copy2(srcname, dstname)
        print("[INFO]:%s" % dstname)

    copystat(src, dst)


if __name__ == '__main__':
    execute()



import logging
import yaml
import logging.config
import os


def setup_logging(default_path='log.yaml', default_level=logging.INFO):
    path = default_path
    if os.path.exists(path):
        with open(path, 'r', encoding='utf-8') as f:
            config = yaml.load(f)
            logging.config.dictConfig(config)
    else:
        logging.basicConfig(level=default_level)



# coding: utf-8
import pika
import json
import logging
import threading
from datetime import datetime, date
from bson import ObjectId
from pika.exceptions import AMQPConnectionError

logger = logging.getLogger(__name__)


class ExchangeTypes():
    DEFAULT = 'default'
    DIRECT = "direct"
    FANOUT = "fanout"
    TOPIC = 'topic'


class CJsonEncoder(json.JSONEncoder):
    def default(self, obj):
        if hasattr(obj, 'keys') and hasattr(obj, '__getitem__'):
            return dict(obj)
        elif isinstance(obj, datetime):
            return obj.strftime('%Y-%m-%dT%H:%M:%S')
        elif isinstance(obj, date):
            # return obj.strftime('%Y-%m-%d')
            return obj.strftime('%Y-%m-%dT%H:%M:%S')  # mongodb的date类型为python的datetime类型
        elif isinstance(obj, ObjectId):
            return str(obj)
        else:
            return json.JSONEncoder.default(self, obj)


class AMQPClient():
    """
    定义可连接至RabbitMQ的基类
    """
    _connection = None
    host = 'localhost'
    channel = None
    exchange = None
    queue = None
    exchange_type = ExchangeTypes.DEFAULT

    def create_connection_parameters(self):
        return pika.ConnectionParameters(host=self.host)

    def create_connection(self):
        """
        创建Rabbit连接的工厂方法，默认使用 SelectionConnection自动选择连接
        :param kwargs:
        :return:
        """
        return pika.BlockingConnection(self.create_connection_parameters())

    def connect(self):
        self._connection = self.create_connection()
        self.on_connection_open()
        self.channel = self._connection.channel()
        self.setup_exchange()

    def reconnect(self):
        if self._connection is None or self._connection.is_closed:
            self.connect()

    def close(self):
        if self._connection is not None and self._connection.is_open:
            self._connection.close()
            self.on_connection_close()

    def on_connection_open(self):
        pass

    def on_connection_close(self):
        pass

    def setup_exchange(self):
        self.channel.exchange_declare(exchange=self.exchange,
                                      exchange_type=self.exchange_type)

    def setup_queue(self, **kwargs):
        result = self.channel.queue_declare(**kwargs)
        # result = self.channel.queue_declare(exclusive=True)
        self.queue = result.method.queue


class TopicAMQPClient(AMQPClient):
    exchange_type = ExchangeTypes.TOPIC

    def __init__(self, exchange):
        """
        :param exchange: 主题交换机名称
        :param topic: 主题名
        """
        if exchange is None or exchange == '':
            raise ValueError("主题交换机名称不能为空")

        self.exchange = exchange


class TopicProducer(TopicAMQPClient):
    """
    主题消息生产者（发送方)
    """

    def emit(self, topic, message):
        """
        向主题交换机发送消息
        :param message:
        :return:
        """
        if message == '' or message is None:
            raise ValueError("不能发送空内容")

        self.reconnect()

        if self.channel is None:
            raise RuntimeError("消息队列通道尚未初始化")

        self.channel.basic_publish(exchange=self.exchange,
                                   routing_key=topic,
                                   body=message)
        self.close()

    def emit_json(self, topic, message):
        self.emit(topic, json.dumps(message, cls=CJsonEncoder))


class TopicConsumer(TopicAMQPClient):
    """
    主题消息消费方（接收方）
    """
    _callbacks = []
    _background = None

    def __call__(self, topic, auto_ack=True):

        def _(func):
            self._callbacks.append((topic, auto_ack, func))

        return _

    def __del__(self):
        self.stop()

    def start(self):
        self.connect()

        self.setup_queue(queue='', exclusive=True)

        topics = set([t for t, _, f in self._callbacks])

        for topic in topics:
            self.channel.queue_bind(exchange=self.exchange,
                                    queue=self.queue,
                                    routing_key=topic)

        for _, ack, _cb in self._callbacks:
            self.channel.basic_consume(on_message_callback=_cb,
                                       queue=self.queue,
                                       auto_ack=ack)

        self._background = threading.Thread(target=self._start_consuming)
        self._background.start()

    def _start_consuming(self):
        self.channel.start_consuming()

    def stop(self):
        if self._background is not None:
            self.close()
            self._background.join()


class RabbitMQ():
    consumers = []

    def __init__(self, app=None, consumer=None):
        self.host = None
        self.username = None
        self.password = None

        if app is not None:
            self.init(app)
            self.valid_config()

        if consumer is not None:
            self.register_consumer(consumer)

    def init(self, app):
        if app is None:
            raise RuntimeError("App 不能为空")
        self.app = app
        self.config = app.config

    def valid_config(self):
        if not self.config.get('RABBITMQ_HOST'):
            raise Exception("The rabbitMQ application must configure host.")
        self.host = self.config.get('RABBITMQ_HOST')
        self.username = self.config.get('RABBITMQ_USERNAME')
        self.password = self.config.get('RABBITMQ_PASSWORD')

    def register_consumer(self, consumer):
        self.consumers.append(consumer)

    def send(self, exchange, topic, message):
        producer = TopicProducer(exchange=exchange)
        producer.emit(topic, message)

    def run(self):
        [consumer.start() for consumer in self.consumers]



# coding:utf-8
import time
import jwt, datetime
import json
import base64


# from werkzeug.exceptions import BadRequest

def generate_access_token(client_id, client_secret, username, exp=86400, **kwargs):
    exp = time.time() + exp
    payload = {
        'exp': exp,
        'iat': time.time(),
        'iss': client_id,
        'sub': username,
        'aud': 'smartracing.cn',
    }
    if kwargs is not None:
        payload.update(kwargs)

    access_token = jwt.encode(payload, client_secret, algorithm='HS256', headers={'kid': str(client_id)})
    return {
        'token_type': 'Bearer',
        'access_token': str(access_token, 'utf-8'),
        'expires_in': exp
    }


def read_token(token, client_secret):
    """
    从Json Web Token中将 client_id, username, pid, bid, uid 读出
    :param req:
    :return:
    """
    header = jwt.get_unverified_header(token)
    # 不验证签名就解开claims
    claims = jwt.decode(token,
                        key=client_secret,
                        options={'verify_aud': False})
    return {
        'client_id': header.get('kid'),
        'username': claims.get('sub'),
        'pid': claims.get('pid'),
        'bid': claims.get('bid'),
        'uid': claims.get('uid'),
        'mid': claims.get('mid')
    }


def decode_token(token):
    '''
    直接解包jwt，而不做client_secret验证
    '''
    data = json.loads(
        str(base64.urlsafe_b64decode(
            token.split(".")[1] + '=' * (4 - len(token.split(".")[1]) % 4)
        ), "utf-8")
    )
    return {
        'username': data.get('sub'),
        'pid': data.get('pid'),
        'bid': data.get('bid'),
        'uid': data.get('uid'),
        'mid': data.get('mid')
    }



# -*- coding: utf-8 -*-

from setuptools import setup, find_packages
import os

dir = os.path.abspath(os.path.dirname(__file__))

with open(os.path.join(dir, 'requirements.txt')) as reqs_file:
    REQS = reqs_file.read()

with open('README.md', encoding='utf-8') as readme_file:
    README = readme_file.read()

setup(
    name='pytitan',
    version='1.0.6',
    packages=find_packages(exclude=["tests"]),
    install_requires=REQS,
    include_package_data=True,
    package_data={
        '': ['templates/*.*',
             'templates/Dockerfile',
             'templates/.dockerignore',
             'templates/.gitignore',
             'templates/**/*.*']
    },
    url='http://120.77.169.160:10001/python/pytitan',
    license='BSD',
    author='Ray',
    entry_points={
        'console_scripts': ['titan = titan:execute']
    },
    author_email='csharp2002@hotmail.com',
    description="The computing framework for microservice.",
    long_description=README,
    zip_safe=False,
    platforms='any',
    keywords=['titan', 'microservice', 'ynkg', 'newland'],
    classifiers=['Development Status :: 4 - Beta',
                 'Intended Audience :: Developers',
                 'License :: OSI Approved :: BSD License',
                 'Natural Language :: English',
                 'Operating System :: OS Independent',
                 'Programming Language :: Python :: 3.6',
                 'Topic :: Software Development :: Libraries',
                 'Topic :: Utilities'])



# coding = utf8
import json

import requests


def send_msg(mobile, title, body, url="http://127.0.0.1:8001/v1/sms/send", user="znsy", pwd="ynkg2019"):
    """
    向短信网关发送SMS信息
    :param user:
    :param title:
    :param pwd:
    :param mobile:
    :param body:
    :param url:
    :return:
    """
    if title == '' or title is None:
        raise ValueError(u'短信标题不能为空！')

    if body == '' or body is None:
        raise ValueError(u'短信内容不能空!')

    response = requests.post(url=url, data={
        'user': user,
        'pwd': pwd,
        'number': mobile,
        'content': "【%s】%s" % (title, body)
    })
    result = json.loads(response.text)
    return result
